@@ -1,49 +1,48 @@
 import re
-from typing import List, Optional, Union, Tuple
+from typing import List, Optional, Tuple, Union
 
-from easytxt import html
-from easytxt import sentences
+from easytxt import html, sentences
 from easytxt import text as utext
 
 
 class TextParser:
-    cached_sentences = []
-    cached_features = []
+    cached_sentences: List[str] = []
+    cached_features: List[Union[str, Tuple[str, str]]] = []
 
     def __init__(
-            self,
-            text: Optional[Union[str, float, int]] = None,
-            language: str = 'en',
-            css_query: Optional[str] = None,
-            exclude_css: Optional[Union[List[str], str]] = None,
-            allow: Optional[Union[str, List[str]]] = None,
-            callow: Optional[Union[str, List[str]]] = None,
-            from_allow: Optional[Union[str, List[str]]] = None,
-            from_callow: Optional[Union[str, List[str]]] = None,
-            to_allow: Optional[Union[str, List[str]]] = None,
-            to_callow: Optional[Union[str, List[str]]] = None,
-            deny: Optional[Union[str, List[str]]] = None,
-            cdeny: Optional[Union[str, List[str]]] = None,
-            normalize: bool = True,
-            capitalize: bool = True,
-            title: bool = False,
-            uppercase: bool = False,
-            lowercase: bool = False,
-            min_chars: int = 5,
-            replace_keys: Optional[list] = None,
-            remove_keys: Optional[list] = None,
-            replace_keys_raw_text: Optional[list] = None,
-            remove_keys_raw_text: Optional[list] = None,
-            split_inline_breaks: bool = True,
-            inline_breaks: Optional[List[str]] = None,
-            merge_sentences: bool = True,
-            stop_key: str = '.',
-            stop_keys_split: Optional[List[str]] = None,
-            stop_keys_ignore: Optional[List[str]] = None,
-            sentence_separator: str = ' ',
-            feature_split_keys: Optional[List[str]] = None,
-            text_num_to_numeric: bool = False,
-            autodetect_html: bool = True
+        self,
+        text: Optional[Union[str, float, int]] = None,
+        language: str = "en",
+        css_query: Optional[str] = None,
+        exclude_css: Optional[Union[List[str], str]] = None,
+        allow: Optional[Union[str, List[str]]] = None,
+        callow: Optional[Union[str, List[str]]] = None,
+        from_allow: Optional[Union[str, List[str]]] = None,
+        from_callow: Optional[Union[str, List[str]]] = None,
+        to_allow: Optional[Union[str, List[str]]] = None,
+        to_callow: Optional[Union[str, List[str]]] = None,
+        deny: Optional[Union[str, List[str]]] = None,
+        cdeny: Optional[Union[str, List[str]]] = None,
+        normalize: bool = True,
+        capitalize: bool = True,
+        title: bool = False,
+        uppercase: bool = False,
+        lowercase: bool = False,
+        min_chars: int = 5,
+        replace_keys: Optional[list] = None,
+        remove_keys: Optional[list] = None,
+        replace_keys_raw_text: Optional[list] = None,
+        remove_keys_raw_text: Optional[list] = None,
+        split_inline_breaks: bool = True,
+        inline_breaks: Optional[List[str]] = None,
+        merge_sentences: bool = True,
+        stop_key: str = ".",
+        stop_keys_split: Optional[List[str]] = None,
+        stop_keys_ignore: Optional[List[str]] = None,
+        sentence_separator: str = " ",
+        feature_split_keys: Optional[List[str]] = None,
+        text_num_to_numeric: bool = False,
+        autodetect_html: bool = True,
     ):
 
         self._text = text
@@ -113,7 +112,7 @@ class TextParser:
     def text(self) -> str:
         return sentences.to_text(
             sentences=self.sentences,
-            separator=self._sentence_separator
+            separator=self._sentence_separator,
         )
 
     @property
@@ -148,7 +147,7 @@ class TextParser:
             if re.search(
                 pattern=key,
                 string=feature_key,
-                flags=re.IGNORECASE
+                flags=re.IGNORECASE,
             ):
 
                 return feature_value
@@ -165,7 +164,7 @@ class TextParser:
         return None
 
     @property
-    def raw_features(self) -> List[str]:
+    def raw_features(self) -> List[Union[str, Tuple[str, str]]]:
         if self.cached_features:
             return self.cached_features
 
@@ -176,7 +175,7 @@ class TextParser:
         for raw_sentence in raw_sentences:
             raw_feature = utext.to_feature(
                 text=raw_sentence,
-                split_keys=self._feature_split_keys
+                split_keys=self._feature_split_keys,
             )
 
             raw_features.append(raw_feature)
@@ -197,26 +196,26 @@ class TextParser:
         if self._merge_sentences:
             raw_sentences = sentences.merge(
                 sentences=raw_sentences,
-                stop_keys_ignore=self._stop_keys_ignore
+                stop_keys_ignore=self._stop_keys_ignore,
             )
 
         if self._replace_keys:
             raw_sentences = sentences.replace_chars_by_keys(
                 sentences=raw_sentences,
-                replace_keys=self._replace_keys
+                replace_keys=self._replace_keys,
             )
 
         if self._remove_keys:
             raw_sentences = sentences.remove_chars_by_keys(
                 sentences=raw_sentences,
-                remove_keys=self._remove_keys
+                remove_keys=self._remove_keys,
             )
 
         raw_sentences = self._min_chars_limit(raw_sentences)
 
         raw_sentences = sentences.add_stop(
             sentences=raw_sentences,
-            stop_key=self._stop_key
+            stop_key=self._stop_key,
         )
 
         raw_sentences = self._sentences_manage_case(raw_sentences)
@@ -234,15 +233,13 @@ class TextParser:
         html_raw_sentences = html.to_sentences(
             html_text=raw_text,
             css_query=self._css_query,
-            exclude_css=self._exclude_css
+            exclude_css=self._exclude_css,
         )
 
-        raw_sentences = []
+        raw_sentences: List[str] = []
 
         for html_raw_sentence in html_raw_sentences:
-            raw_sentences += self._text_to_raw_sentences(
-                html_raw_sentence
-            )
+            raw_sentences += self._text_to_raw_sentences(html_raw_sentence)
 
         return raw_sentences
 
@@ -257,7 +254,7 @@ class TextParser:
             stop_keys=self._stop_keys_split,
             split_inline_breaks=self._split_inline_breaks,
             inline_breaks=self._inline_breaks,
-            min_chars=self._min_chars
+            min_chars=self._min_chars,
         )
 
     def _filter_raw_sentences(self, raw_sentences: List[str]) -> List[str]:
@@ -267,7 +264,7 @@ class TextParser:
             raw_sentences = sentences.allow_contains(
                 sentences=raw_sentences,
                 keys=allow_keys,
-                case_sensitive=bool(self._callow)
+                case_sensitive=bool(self._callow),
             )
 
         from_allow_keys = self._from_allow or self._from_callow
@@ -276,7 +273,7 @@ class TextParser:
             raw_sentences = sentences.from_allow_contains(
                 sentences=raw_sentences,
                 keys=from_allow_keys,
-                case_sensitive=bool(self._from_callow)
+                case_sensitive=bool(self._from_callow),
             )
 
         to_allow_keys = self._to_allow or self._to_callow
@@ -285,7 +282,7 @@ class TextParser:
             raw_sentences = sentences.to_allow_contains(
                 sentences=raw_sentences,
                 keys=to_allow_keys,
-                case_sensitive=bool(self._to_callow)
+                case_sensitive=bool(self._to_callow),
             )
 
         deny_keys = self._cdeny or self._deny
@@ -294,7 +291,7 @@ class TextParser:
             raw_sentences = sentences.deny_contains(
                 sentences=raw_sentences,
                 keys=deny_keys,
-                case_sensitive=bool(self._cdeny)
+                case_sensitive=bool(self._cdeny),
             )
 
         return raw_sentences
@@ -324,8 +321,8 @@ class TextParser:
         return raw_sentences
 
     def _convert_text_num_to_numeric_in_sentences(
-            self,
-            raw_sentences: List[str]
+        self,
+        raw_sentences: List[str],
     ) -> List[str]:
 
         return [
@@ -337,13 +334,13 @@ class TextParser:
         if self._replace_keys_raw_text:
             raw_text = utext.replace_chars_by_keys(
                 text=raw_text,
-                replace_keys=self._replace_keys_raw_text
+                replace_keys=self._replace_keys_raw_text,
             )
 
         if self._remove_keys_raw_text:
             raw_text = utext.remove_chars_by_keys(
                 text=raw_text,
-                remove_keys=self._remove_keys_raw_text
+                remove_keys=self._remove_keys_raw_text,
             )
 
         return raw_text
